﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;
using Bee.Caching;
using Bee.Data;
using Bee;
using Bee.Util;
using Bee.Service;

namespace Bee.BLL
{

    public class OnlineUserService : BaseRunService
    {
        public OnlineUserService()
        {
            Interval = 60;// second
        }
        protected override void Run()
        {
            OnlineUserManager.Instance.CheckOnline();
            DateTime now = DateTime.Now;
            OnlineUserManager.Instance.CurrentTick = now.Ticks; 
        }
    }

    public class OnlineUserManager
    {
        private static OnlineUserManager instance = new OnlineUserManager();

        private Dictionary<int, long> userDict = new Dictionary<int, long>();
        private long currentTick = 0;

        private List<int> onlineUserList = new List<int>();

        private OnlineUserManager()
        {
            ServiceManager.Instance.AppendTask(new OnlineUserService());
        }

        public static OnlineUserManager Instance
        {
            get
            {
                return instance;
            }
        }

        public void TickUserId(int userId)
        {
            if (userDict.ContainsKey(userId))
            {
                userDict[userId] = currentTick;
            }
            else
            {
                lock (this)
                {
                    userDict.Add(userId, currentTick);
                }
            }
        }

        public void RemoveUserId(int userId)
        {
            lock (this)
            {
                userDict.Remove(userId);
            }
        }

        public List<int> OnlineUserList
        {
            get
            {
                return onlineUserList;
            }
        }

        public void CheckOnline()
        {
            List<int> remainList = new List<int>();
            List<int> removingList = new List<int>();
            foreach (int userId in userDict.Keys)
            {
                if (userDict[userId] < currentTick)
                {
                    removingList.Add(userId);
                }
                else
                {
                    remainList.Add(userId);
                }
            }

            lock (this)
            {
                foreach (int userId in removingList)
                {
                    userDict.Remove(userId);
                }
            }

            onlineUserList = remainList;
        }

        public long CurrentTick
        {
            get
            {
                return currentTick;
            }
            set
            {
                currentTick = value;
            }
        }
        
    }
}
