﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Bee.Web;
using Bee.Models;
using Bee.Data;
using System.Data;
using Bee.Util;
using Bee.Caching;

namespace Bee.Security.Controllers
{
    public class AuthGroupController : AuthControllerBase<AuthGroup>
    {
        static AuthGroupController()
        {
            DataMapping.Instance.Register("AuthGroupMapping", () => {
                using (DbSession dbSession = new DbSession(ConfigUtil.GetAppSettingValue<string>(Constants.AuthDbConnectionKey)))
                {
                    return dbSession.Query("AuthGroup", null);
                }
            });
        }

        public override PageResult Index(BeeDataAdapter dataAdapter)
        {
            DataTable result;
            using (DbSession dbSession = GetDbSession())
            {
                result = dbSession.Query(TableName, SqlCriteria.New.Equal("grouptype", 1).Equal("delflag", 0));

                ViewData.Add("roleinfo", dbSession.Query("AuthRole", SqlCriteria.New.Equal("delflag", false)));
            }

            return View(ControllerName, "Index", result);
        }

        public virtual void Save(AuthGroup authGroup, string roleInfo)
        {
            using (DbSession dbSession = GetDbSession(true))
            {
                ThrowExceptionUtil.ArgumentNotNullOrEmpty(authGroup.Name, "Name");

                SqlCriteria sqlCriteria = SqlCriteria.New.Equal("name", authGroup.Name);
                if(authGroup.Id > 0)
                {
                    sqlCriteria.NotEqual("id", authGroup.Id);
                }

                ThrowExceptionUtil.ArgumentConditionTrue(!dbSession.IsExist("AuthGroup", sqlCriteria)
                    , "Name", "存在相同名字");

                if (authGroup.GroupType == 0)
                {
                    authGroup.GroupType = 1;
                }

                if (authGroup.Id == 0)
                {
                    authGroup.GroupCode = GetGroupCode(authGroup);
                }
                dbSession.Save(authGroup);

                int groupId = authGroup.Id;
                dbSession.Delete<AuthGroupRole>(SqlCriteria.New.Equal("groupid", groupId));

                List<int> groupRoleList = (from item in roleInfo.Split(new Char[] { ',' },
                                              StringSplitOptions.RemoveEmptyEntries)
                                           select int.Parse(item)).ToList();

                AuthGroupRole authGroupRole = new AuthGroupRole();
                authGroupRole.GroupId = groupId;
                foreach (int roleId in groupRoleList)
                {
                    authGroupRole.RoleId = roleId;
                    dbSession.Insert(authGroupRole);
                }

                dbSession.CommitTransaction();
            }

            CacheManager.Instance.RemoveCategoryCache(Constants.UserAllPermission);
        }

        public virtual BeeDataAdapter Detail(int id)
        {
            BeeDataAdapter dataAdapter;
            AuthGroup result = null;

            using (DbSession dbSession = GetDbSession())
            {
                result = dbSession.Query<AuthGroup>(SqlCriteria.New.Equal("id", id)).FirstOrDefault();

                List<AuthGroupRole> groupRoleList =
                       dbSession.Query<AuthGroupRole>(SqlCriteria.New.Equal("groupid", id));

                string groupRole = string.Join(",", (from item in groupRoleList
                                                     select item.RoleId.ToString()).ToArray());

                dataAdapter = BeeDataAdapter.From(result);
                dataAdapter.Add("roleinfo", groupRole);
            }

            return dataAdapter;
        }

        private string GetGroupCode(AuthGroup authGroup)
        {
            string result = string.Empty;
            DataTable dataTable = null;

            dataTable = DbSession.Current.QueryTop("AuthGroup", SqlCriteria.New.NotEqual("id", authGroup.Id).Equal("parentid", authGroup.ParentId), "id desc", 1);
            if (dataTable.Rows.Count > 0)
            {
                string orgCode = dataTable.Rows[0]["groupcode"].ToString();
                int length = orgCode.Length;
                int index = int.Parse(orgCode.Substring(length - 3, 3));
                index++;
                if (length > 3)
                {
                    result = orgCode.Substring(0, length - 3) + "{0:D3}".FormatWith(index);
                }
                else
                {
                    result = "{0:D3}".FormatWith(index);
                }
            }
            else
            {
                if (authGroup.ParentId == 0)
                {
                    result = "001";
                }
                else
                {
                    dataTable = DbSession.Current.QueryTop("AuthGroup", SqlCriteria.New.Equal("id", authGroup.ParentId), "id desc", 1);
                    string orgCode = dataTable.Rows[0]["groupcode"].ToString();
                    result = "{0}001".FormatWith(orgCode);
                }
            }

            return result;
        }
    }
}
